<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Product Export Entity for Magento 2 (System)
 */

namespace Amasty\ProductExportEntity\Export\DataHandling\FieldGenerator;

use Amasty\ExportCore\Api\VirtualField\GeneratorInterface;
use Magento\Framework\App\ResourceConnection;

class Ratings implements GeneratorInterface
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    public function __construct(
        ResourceConnection $resourceConnection
    ) {
        $this->resourceConnection = $resourceConnection;
    }

    public function generateValue(array $currentRecord): float
    {
        $value = 0.0;
        if (isset($currentRecord['review_id']) && $votes = $this->getVotesByReview((int)$currentRecord['review_id'])) {
            $value = array_sum($votes) / count($votes);
        }

        return round($value, 1);
    }

    private function getVotesByReview(int $reviewId): array
    {
        $connection = $this->resourceConnection->getConnection();
        $voteTable = $this->resourceConnection->getTableName('rating_option_vote');
        $select = $connection->select()
            ->from(['v' => $voteTable], ['value'])
            ->where('v.review_id = ? ', $reviewId);

        return $connection->fetchCol($select);
    }
}
