<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Product Review Export
 */

namespace Amasty\ProductReviewExport\Setup\Operation;

use Amasty\ExportPro\Model\Job\ResourceModel\Job;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Filesystem\Io\File;
use Magento\Framework\Module\Dir;
use Magento\Framework\Module\Dir\Reader;

class AddProfile
{
    public const CONFIG = '/Fixtures/config.json';
    public const PROFILE_DATA = [
        'title' => 'Google Product Review',
        'entity_code' => 'catalog_product_entity_review',
    ];

    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * @var Reader
     */
    private $reader;

    /**
     * @var File
     */
    private $file;

    public function __construct(
        ResourceConnection $connection,
        Reader $reader,
        File $file
    ) {
        $this->resourceConnection = $connection;
        $this->reader = $reader;
        $this->file = $file;
    }

    public function saveProfile(): void
    {
        $resultConfig = array_merge(self::PROFILE_DATA, ['config' => $this->getConfig()]);
        $connection = $this->resourceConnection->getConnection();
        $exportCronJobTable = $this->resourceConnection->getTableName(Job::TABLE_NAME);

        $connection->insert($exportCronJobTable, $resultConfig);
    }

    private function getConfig(): string
    {
        $moduleDir = $this->reader->getModuleDir(Dir::MODULE_SETUP_DIR, 'Amasty_ProductReviewExport');
        $configPath = $moduleDir . self::CONFIG;
        if (!$this->file->fileExists($configPath)) {
            throw new LocalizedException(__('Config file does not exist'));
        }

        return $this->file->read($configPath);
    }
}
