<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="html" indent="yes"/>

    <xsl:template match="/reviews">
        <feed xmlns:vc="http://www.w3.org/2007/XMLSchema-versioning"
              xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
              xsi:noNamespaceSchemaLocation="http://www.google.com/shopping/reviews/schema/product/2.3/product_reviews.xsd">
            <version>2.3</version>
            <aggregator>
                <name>Sample Reviews Aggregator (if applicable)</name>
            </aggregator>
            <publisher>
                <name>Sample Retailer</name>
                <favicon>http://www.example.com/favicon.png</favicon>
            </publisher>
            <reviews>
                <xsl:apply-templates select="review"/>
            </reviews>
        </feed>
    </xsl:template>

    <xsl:template match="review">
        <review>
            <xsl:if test="string-length(review_id) != 0">
                <review_id>
                    <xsl:value-of select="review_id"/>
                </review_id>
            </xsl:if>
            <reviewer>
                <name>
                    <xsl:value-of select="nickname"/>
                </name>
                <xsl:if test="string-length(customer_id) != 0">
                    <reviewer_id>
                        <xsl:value-of select="customer_id"/>
                    </reviewer_id>
                </xsl:if>
            </reviewer>
            <review_timestamp>
                <xsl:value-of select="created_at"/>
            </review_timestamp>
            <xsl:if test="string-length(title) != 0">
                <title>
                    <xsl:value-of select="title"/>
                </title>
            </xsl:if>
            <content>
                <xsl:value-of select="detail"/>
            </content>
            <review_url>
                <xsl:value-of select="concat(catalog_product_entity/review/catalog_product_attribute/review/product_url, '#reviews')" />
            </review_url>
            <ratings>
                <overall min="1" max="5">
                    <xsl:value-of select="ratings"/>
                </overall>
            </ratings>
            <products>
                <xsl:apply-templates select="catalog_product_entity/review/catalog_product_attribute/review"/>
            </products>
            <is_spam>false</is_spam>
        </review>
    </xsl:template>

    <xsl:template match="catalog_product_entity/review/catalog_product_attribute/review">
        <xsl:apply-templates select="catalog_product_attribute/review"/>
    </xsl:template>

    <xsl:template match="catalog_product_attribute/review">
        <product>
            <product_ids>
                <xsl:if test="string-length(upc) != 0">
                    <gtins>
                        <gtin>
                            <xsl:value-of select="upc"/>
                        </gtin>
                    </gtins>
                </xsl:if>
                <xsl:if test="string-length(mpn) != 0">
                    <mpns>
                        <mpn>
                            <xsl:value-of select="mpn"/>
                        </mpn>
                    </mpns>
                </xsl:if>
                <xsl:if test="string-length(../../sku) != 0">
                    <skus>
                        <sku>
                            <xsl:value-of select="../../sku"/>
                        </sku>
                    </skus>
                </xsl:if>
                <xsl:if test="string-length(brand) != 0">
                    <brands>
                        <brand>
                            <xsl:value-of select="brand"/>
                        </brand>
                    </brands>
                </xsl:if>
                <xsl:if test="string-length(asin) != 0">
                    <asins>
                        <asin>
                            <xsl:value-of select="asin"/>
                        </asin>
                    </asins>
                </xsl:if>
            </product_ids>
            <xsl:if test="string-length(name) != 0">
                <product_name>
                    <xsl:value-of select="name"/>
                </product_name>
            </xsl:if>
            <product_url>
                <xsl:value-of select="product_url"/>
            </product_url>
        </product>
    </xsl:template>

</xsl:stylesheet>
